"""
DCOP interface to kbtv
"""

## MODULE IMPORTS ########################################################

import btcopyright
from dcopexport import DCOPExObj

## MODULE COPYRIGHT ######################################################

MODULE_AUTHOR       = btcopyright.MY_NAME
MODULE_AUTHOR_EMAIL = btcopyright.MY_EMAIL
MODULE_COPYRIGHT    = btcopyright.MY_COPYRIGHT
MODULE_LICENSE      = btcopyright.BSD_LICENSE
MODULE_LICENSE_TEXT = btcopyright.BSD_LICENSE_TEXT

## KBTVDCOPEXOBJ CLASS ###################################################

class KbtvDCOPExObj(DCOPExObj):
    """
    Provides a DCOP interface to kbtv. Exports all actions and various
    "get and set" methods. Inherits DCOPExObj.
    """
    def __init__(self, obj, id):
        """
        -> KbtvDCOPExObj
        
        The constructor takes the argument obj, which should be the
        application's main part/window and the application ID. Most 
        but not all exported method names are identical to the actual 
        methods' names.
        """
        DCOPExObj.__init__(self, id)
        self.addMethod("void tuneTo(int)", obj.tuneTo)
        self.addMethod("void tuneUp()", obj.tuneUp)
        self.addMethod("void tuneDown()", obj.tuneDown)
        self.addMethod("void tiltUp()", obj.tiltUp)
        self.addMethod("void tiltDown()", obj.tiltDown)
        self.addMethod("void panLeft()", obj.panLeft)
        self.addMethod("void panRight()", obj.panRight)
        self.addMethod("void quit()", obj.quit)
        self.addMethod("void channelEditor()", obj.editChans)
        self.addMethod("void hardwareInfo()", obj.hwInfo)
        self.addMethod("void pictureSettings()", obj.setPicture)
        self.addMethod("void setBrightness(int)", obj.setBrightness)
        self.addMethod("void setContrast(int)", obj.setContrast)
        self.addMethod("void setColor(int)", obj.setColor)
        self.addMethod("void setSaturation(int)", obj.setSaturation)
        self.addMethod("void mixer1()", obj.mixer1)
        self.addMethod("void mixer2()", obj.mixer2)
        self.addMethod("void mixer3()", obj.mixer3)
        self.addMethod("void mixer4()", obj.mixer4)
        self.addMethod("void mixer5()", obj.mixer5)
        self.addMethod("void mixer6()", obj.mixer6)
        self.addMethod("void setVolume(int)", obj.setVolume)
        self.addMethod("void volumeUp()", obj.volumeUp)
        self.addMethod("void volumeDown()", obj.volumeDown)
        self.addMethod("void driverBktr()", obj.driverBktr)
        self.addMethod("void driverSaa()", obj.driverSaa)
        self.addMethod("void driverPwc()", obj.driverPwc)
        self.addMethod("void sizeSmall()", obj.sizeSmall)
        self.addMethod("void sizeMedium()", obj.sizeMedium)
        self.addMethod("void sizeLarge()", obj.sizeLarge)
        self.addMethod("void fullScreen()", obj.fullScreen)
        self.addMethod("void pause()", obj.pause)
        self.addMethod("int currentChannel()", obj.currentChannel)
        self.addMethod("int brightness()", obj.brightness)
        self.addMethod("int contrast()", obj.contrast)
        self.addMethod("int color()", obj.color)
        self.addMethod("int saturation()", obj.saturation)
        self.addMethod("int volume()", obj.volume)
        self.addMethod("QString currentStatus()", obj.currentStatus)
        self.addMethod("QString driver()", obj.driver)

## END ###################################################################

if __name__ == "__main__":
    print "This is a module. Import it instead."
